unit MainControlling;
{$MODE ObjFpc}
{$H+}
{$INTERFACES Corba}

interface

uses
  Classes, SysUtils,
  (* project options *)
  Model, Definitions;

type
  TMainController = class(TObject)
    private
      Dao: IPersistence;
      MainView: IMainView;
      DetailView: IDetailView;
    public
      constructor Create(ADao: IPersistence; AMainView: IMainView; ADetailView: IDetailView);
    public
      procedure CreatePerson;
      procedure EditPerson(PersonHead: TPersonHead);
      procedure ShowDetails(PersonHead: TPersonHead);
      procedure SyncMainview;
  end;

implementation

constructor TMainController.Create(ADao: IPersistence; AMainView: IMainView; ADetailView: IDetailView);
begin
  inherited Create;
  Dao := ADao;
  MainView := AMainView;
  DetailView := ADetailView;
end;

procedure TMainController.SyncMainview;
var
  PersonHeads: TPersonHeadList;
begin
  PersonHeads := Dao.GetAllPersonHeads;
  MainView.BindPersonHeads(PersonHeads);
end;

procedure TMainController.CreatePerson;
var
  NewPerson: TPerson = nil;
begin
  try
    NewPerson := TPerson.Create;
    if DetailView.NewPerson(NewPerson) then begin
      Dao.InsertPerson(NewPerson);
      SyncMainview;
    end;
  finally
    FreeAndNil(NewPerson);
  end;
end;

procedure TMainController.EditPerson(PersonHead: TPersonHead);
var
  Person: TPerson = nil;
begin
  try
    Person := Dao.GetPerson(PersonHead.Id);
    if Assigned(Person) then begin
      if DetailView.EditPerson(Person) then begin
        Dao.UpdatePerson(Person);
        SyncMainview;
      end;
    end;
  finally
    FreeAndNil(Person);
  end;
end;

procedure TMainController.ShowDetails(PersonHead: TPersonHead);
var
  Person: TPerson = nil;
begin
  Person := Dao.GetPerson(PersonHead.Id);
  if Assigned(Person) then begin
    DetailView.ShowPerson(Person);
  end;
end;

end.
