unit Model;
{$MODE ObjFpc}
{$H+}
{$M+}

interface

uses
  Classes, SysUtils, Contnrs;

type
  TPersonHead = class(TObject)
    private
      FFirstname: String;
      FId: LongInt;
      FLastname: String;
    published
      property Id: LongInt read FId write FId;
      property Firstname: String read FFirstname write FFirstname;
      property Lastname: String read FLastname write FLastname;
  end;

  TPersonHeadList = class;

  TPersonHeadListEnumerator = class(TObject)
    private
      FList: TPersonHeadList;
      FPosition: Integer;
    public
      constructor Create(AList: TPersonHeadList);
      function GetCurrent: TPersonHead;
      function MoveNext: Boolean;
      property Current: TPersonHead read GetCurrent;
  end;

  TPersonHeadList = class(TObjectList)
    protected
      function GetItem(Index: Integer): TPersonHead;
      procedure SetItem(Index: Integer; AObject: TPersonHead);
    public
      function Add(AObject: TPersonHead): Integer;
      function Extract(Item: TPersonHead): TPersonHead;
      function Remove(AObject: TPersonHead): Integer;
      function IndexOf(AObject: TPersonHead): Integer;
      procedure Insert(Index: Integer; AObject: TPersonHead);
      function First: TPersonHead;
      function Last: TPersonHead;
      function GetEnumerator: TPersonHeadListEnumerator;
      property Items[Index: Integer]: TPersonHead read GetItem write SetItem; default;
  end;

  TPerson = class(TPersonHead)
    private
      FAdress: String;
      FCity: String;
      FEmail: String;
      FPhone: STring;
      FPostcode: String;
    published
      property Adress: String read FAdress write FAdress;
      property City: String read FCity write FCity;
      property Postcode: String read FPostcode write FPostcode;
      property Email: String read FEmail write FEmail;
      property Phone: STring read FPhone write FPhone;
  end;

implementation

constructor TPersonHeadListEnumerator.Create(AList: TPersonHeadList);
begin
  inherited Create;
  FList := AList;
  FPosition := -1;
end;

function TPersonHeadListEnumerator.GetCurrent: TPersonHead;
begin
  Result := FList[FPosition];
end;

function TPersonHeadListEnumerator.MoveNext: Boolean;
begin
  Inc(FPosition);
  Result := FPosition < FList.Count;
end;

function TPersonHeadList.GetItem(Index: Integer): TPersonHead;
begin
  Result := TPersonHead(inherited GetItem(Index));
end;

procedure TPersonHeadList.SetItem(Index: Integer; AObject: TPersonHead);
begin
  inherited SetItem(Index, AObject);
end;

function TPersonHeadList.Add(AObject: TPersonHead): Integer;
begin
  Result := inherited Add(AObject);
end;

function TPersonHeadList.Extract(Item: TPersonHead): TPersonHead;
begin
  Result := TPersonHead(inherited Extract(Item));
end;

function TPersonHeadList.Remove(AObject: TPersonHead): Integer;
begin
  Result := inherited Remove(AObject);
end;

function TPersonHeadList.IndexOf(AObject: TPersonHead): Integer;
begin
  Result := inherited IndexOf(AObject);
end;

procedure TPersonHeadList.Insert(Index: Integer; AObject: TPersonHead);
begin
  inherited Insert(Index, AObject);
end;

function TPersonHeadList.First: TPersonHead;
begin
  Result := TPersonHead(inherited First);
end;

function TPersonHeadList.Last: TPersonHead;
begin
  Result := TPersonHead(inherited Last);
end;

function TPersonHeadList.GetEnumerator: TPersonHeadListEnumerator;
begin
  Result := TPersonHeadListEnumerator.Create(Self);
end;

end.
