unit Persistence;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils,
  (* project libs *)
  SqliteI, SqliteOrm,
  (* project units *)
  Definitions, Model;

type
  TPersistence = class(TObject, IPersistence)
    private
      Connector: TSqliteConnector;
      Mapper: TSqliteAutoMapper;
    public
      constructor Create(DatabaseFilename: String);
      destructor Destroy; override;
    public
      procedure CreateDatabase;
      function GetAllPersonHeads: TPersonHeadList;
      function GetPerson(Id: LongInt): TPerson;
      procedure InsertPerson(APerson: TPerson);
      procedure UpdatePerson(APerson: TPerson);
  end;

implementation

constructor TPersistence.Create(DatabaseFilename: String);
begin
  inherited Create;
  Connector := TSqliteConnector.Create(DatabaseFilename);
  Mapper := TSqliteAutoMapper.Create;
end;

destructor TPersistence.Destroy;
begin
  FreeAndNil(Mapper);
  FreeAndNil(Connector);
  inherited Destroy;
end;

procedure TPersistence.CreateDatabase;
var
  st: TSqliteStatement;
begin
  try
    st := Connector.Prepare('');
    st.Execute;
  finally
    FreeAndNil(st);
  end;
end;

function TPersistence.GetAllPersonHeads: TPersonHeadList;
var
  st: TSqliteStatement;
begin
  try
    Result := TPersonHeadList.Create();
    st := Connector.Prepare('SELECT Id, Firstname, Lastname FROM Persons');
    Mapper.ExecuteStatementAsList(st, TPersonHead, Result);
  finally
    FreeAndNil(st);
  end;
end;

function TPersistence.GetPerson(Id: LongInt): TPerson;
var
  st: TSqliteStatement;
begin
  try
    st := Connector.Prepare('SELECT * FROM Persons WHERE Id=@Id');
    st.BindParam('@Id', Id);
    Result := Mapper.ExecuteStatementAsObject(st, TPerson) as TPerson;
  finally
    FreeAndNil(st);
  end;
end;

procedure TPersistence.InsertPerson(APerson: TPerson);
var
  st: TSqliteStatement;
begin
  try
    st := Connector.Prepare('INSERT INTO Persons (Id, Firstname, Lastname, Adress, City, Email, Phone, Postcode) ' +
                            'Values (NULL, @Firstname, @Lastname, @Adress, @City, @Email, @Phone, @Postcode)');
    Mapper.BindObjectToParams(st, APerson);
    st.Execute;
  finally
    FreeAndNil(st);
  end;
end;

procedure TPersistence.UpdatePerson(APerson: TPerson);
var
  st: TSqliteStatement;
begin
  try
    st := Connector.Prepare('UPDATE Persons SET Firstname=@Firstname, Lastname=@Lastname, Adress=@Adress, ' +
                            'City=@City, Email=@Email, Phone=@Phone, Postcode=@Postcode WHERE Id=@Id');
    Mapper.BindObjectToParams(st, APerson);
    st.Execute;
  finally
    FreeAndNil(st);
  end;
end;

end.
