unit MainViewing;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
  ExtCtrls,
  (* project units *)
  Model, Definitions, Persistence, MainControlling, PersonDetailsForming;

type
  TMainForm = class(TForm, IMainView)
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    ListBox1: TListBox;
    Panel1: TPanel;
    Splitter1: TSplitter;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    Controller: TMainController;
    Dao: TPersistence;
  private
    procedure ClearListBox;
    procedure BindPersonHeads(List: TPersonHeadList);
  public
    { public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$R *.lfm}

procedure TMainForm.FormCreate(Sender: TObject);
begin
  Dao := TPersistence.Create('data.db');
  Application.CreateForm(TDetailsForm, DetailsForm);
  Controller := TMainController.Create(Dao, Self, DetailsForm);
  Controller.SyncMainview;
end;

procedure TMainForm.Button1Click(Sender: TObject);
begin
  if ListBox1.ItemIndex > -1 then
    Controller.ShowDetails(ListBox1.Items.Objects[ListBox1.ItemIndex] as TPersonHead);
end;

procedure TMainForm.Button2Click(Sender: TObject);
begin
  if ListBox1.ItemIndex > -1 then
    Controller.EditPerson(ListBox1.Items.Objects[ListBox1.ItemIndex] as TPersonHead);
end;

procedure TMainForm.Button3Click(Sender: TObject);
begin
  Controller.CreatePerson;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  ClearListBox;
  FreeAndNil(Controller);
  FreeAndNil(Dao);
end;

procedure TMainForm.ClearListBox;
var
  i: Integer;
begin
  if ListBox1.Count > 0 then begin
    for i := 0 to (ListBox1.Count - 1) do begin
      ListBox1.Items.Objects[i].Free;
      ListBox1.Items.Objects[i] := nil;
    end;
  end;
  ListBox1.Clear;
end;

procedure TMainForm.BindPersonHeads(List: TPersonHeadList);
var
  o: TPersonHead;
begin
  ClearListBox;
  for o in List do begin
    ListBox1.AddItem(o.Lastname + ', ' + o.Firstname, o);
  end;
  List.OwnsObjects := False;
  FreeAndNil(List);
end;

end.
